using System; 
using System.Collections.Generic; 
using System.Diagnostics; 
using System.Windows.Forms; 

public class FavoriteManager<TFavorite> : UserControl 
  where TFavorite : Favorite, new() 
{ 
  private ListBox listBox1; 
  List<TFavorite> favs = new List<TFavorite>(); 

  public delegate void FavoriteEventHandler(FavoriteManager<TFavorite> sender, FavoriteEventArgs<TFavorite> e); 
  public event FavoriteEventHandler FavoriteChanged; 

  public FavoriteManager() 
  { 
    InitializeComponent(); 
    listBox1.DisplayMember = "Name"; 
  } 

  public void Add(string name, string url) 
  { 
    TFavorite fav = new TFavorite(); 
    fav.Name = name; 
    fav.URL = new Uri(url); 
    favs.Add(fav); 
    listBox1.DataSource = favs; 
  } 

  public TFavorite[] Favorites 
  { 
    set 
    { 
      favs.Clear(); 
      favs.AddRange(value); 
      listBox1.DataSource = favs; 
    } 

    get { return favs.ToArray(); } 
  } 

  private void InitializeComponent() 
  { 
      this.listBox1 = new System.Windows.Forms.ListBox(); 
      this.SuspendLayout(); 
      // 
      // listBox1 
      // 
      this.listBox1.Dock = System.Windows.Forms.DockStyle.Fill; 
      this.listBox1.FormattingEnabled = true; 
      this.listBox1.Location = new System.Drawing.Point(0, 0); 
      this.listBox1.Name = "listBox"; 
      this.listBox1.Size = new System.Drawing.Size(150, 147); 
      this.listBox1.TabIndex = 0; 
      this.listBox1.SelectedIndexChanged += new System.EventHandler(this.listBox1_SelectedIndexChanged); 
      // 
      // FavoriteManager 
      // 
      this.Controls.Add(this.listBox1); 
      this.Name = "FavoriteManager"; 
      this.ResumeLayout(false); 
  } 

  private void listBox1_SelectedIndexChanged(object sender, EventArgs e) 
  { 
    if (FavoriteChanged != null) 
    { 
      FavoriteChanged(this, new FavoriteEventArgs<TFavorite>((TFavorite)listBox1.SelectedItem)); 
    } 
  } 
} 